const loaderUtils = require('loader-utils');

module.exports = function() {
  const { activationUrl, ips, hmr } = loaderUtils.getOptions(this) || {};
  const hmrCode = `
if (module.hot) {
  module.hot.dispose(function() {
    if (loopHandle !== null) {
      clearTimeout(loopHandle);
      document.title = originTitle;
    }

    // wait for the module to install
    setTimeout(() => {
      // depressed warning: "unexpected require from disposed module"
      module.hot.active = true;
      resolve(__webpack_require__(module.id));
      module.hot.active = false;
    }, 0);
  });
  module.hot.accept(function() {
    // error handle
  });
}
`.trim();

  const refreshCode = `
window.onbeforeunload = function() {
  document.title = originTitle;
}
`.trim();

  return `
//############################ NOTICE ############################//
//                                                                //
//           This is a placeholder module generated by            //
//                 lazy-compile-webpack-plugin.                   //
//     https://github.com/liximomo/lazy-compile-webpack-plugin    //
//                                                                //
//    The real content is in the corresponding hot-update file.   //
//           You won't see this after the page refresh.           //
//                                                                //
//################################################################//

// @activationUrl ${activationUrl}

// modified from https://matthewrayfield.com/articles/animating-urls-with-javascript-and-emojis
var figures = ['🕐','🕑','🕒','🕓','🕔','🕕','🕖','🕗','🕘','🕙','🕚','🕛'];
var originTitle = document.title;
var loopHandle = null;
function loop() {
  loopHandle = setTimeout(loop, 50);
  document.title = 'Compiling ' + figures[Math.floor((Date.now() / 100) % figures.length)];
}
if (document.title.indexOf('Compiling') !== 0) {
  loop();
}

var requestPromise = new Promise((resolve, reject) => {
  ${hmr ? hmrCode : refreshCode}

  var ips = ${JSON.stringify(ips)};
  var activationUrl = '${activationUrl}';
  ips.forEach(function(ip) {
    var img = new Image();
    img.src = activationUrl.replace('{host}', ip);
  });
});

module.exports = {
  then(resolve) {
    resolve(requestPromise)
  }
}
`.trim();
};
